/*
*	Copyright(c) 2012 - 2022  IntervalZero, Inc.  All rights reserved.
*
*	Module Name:
*		RtNalIGC.h
*
*	Abstract:
*		RTX driver header file for RealTek family Gigabit Ethernet controllers
*
*	Environments:
*		Real-time 64-bit Subsystem (supported configurations: RTSSDebug and RTSSRelease)
*
*	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
*	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
*	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
*	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
*	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
*	CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
*	SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
*	INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
*	CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
*	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
*	POSSIBILITY OF SUCH DAMAGE.
*/

#pragma once
//This define will deprecate all unsupported Microsoft C-runtime functions when compiled under RTSS.
//If using this define, #include <rtapi.h> should remain below all windows headers
//#define UNDER_RTSS_UNSUPPORTED_CRT_APIS

#include <SDKDDKVer.h>

// for RTTCPIP Support
#include <winsock2.h>
#include <ws2tcpip.h>

//#include <stdio.h>
//#include <string.h>
//#include <ctype.h>
//#include <conio.h>
//#include <stdlib.h>
//#include <math.h>
//#include <errno.h>
#include <windows.h>
#include <rtapi.h>      // RTX64 APIs that can be used in real-time or Windows applications.
#include "rtnapi.h"
#include <RtNalApi.h>


#ifdef UNDER_RTSS
#include <rtssapi.h>    // RTX64 APIs that can only be used in real-time applications.
#endif // UNDER_RTSS

#include "if_rtx.h"

#if defined UNDER_RTSS
#if defined RTX64_EXPORTS
#define RTNDPROC __declspec(dllexport)
#else
#define RTNDPROC __declspec(dllimport)
#endif
#else
#if defined DLL64_EXPORTS
#define RTNDPROC __declspec(dllexport)
#else
#define RTNDPROC __declspec(dllimport)
#endif
#endif

//*****************************************************************************
// COMMON PCI/PCIe DEFINES
//
#define PCI_COMMON_HDR_LENGTH (FIELD_OFFSET (PCI_COMMON_CONFIG, DeviceSpecific))
#define PCI_MAX_BUS                         255
#define PCI_MAX_DEVICES                     32
#define PCI_MAX_FUNCTION                    8
#define PCI_INVALID_VENDORID                0xFFFF
#define PCI_BAR0 0
#define PCI_BAR1 1
#define PCI_BAR5 5
#define SUPPORTED_BASECLASS		0x02
#define SUPPORTED_SUBCLASS		0x00
#define SUPPORTED_PROG_INF_REG	0x00
#define SZBUFFER_SIZE 100
#define LOCS 24
#define BUS_LOC 0
#define DEV_LOC 1
#define FUNC_LOC 2

#define PCI_BAR_TYPE_MASK	0x00000001
#define PCI_BAR_TYPE_MEM	0x00000000
#define PCI_BAR_TYPE_IO		0x00000001
#define PCI_BAR_TYPE(a)		((a) & PCI_BAR_TYPE_MASK)

#define PCI_BAR_MEM_TYPE_MASK	0x00000006
#define PCI_BAR_MEM_TYPE_32BIT	0x00000000
#define PCI_BAR_MEM_TYPE_64BIT	0x00000004
#define PCI_BAR_MEM_TYPE(a)	((a) & PCI_BAR_MEM_TYPE_MASK)

#define	PCIR_SUBVEND_0	0x2c
#define	PCIR_SUBDEV_0	0x2e

#define SMALL_PAGE_SIZE 4096

#define NANOSEC_PER_SEC	1000000000ULL

/*********************************************************************
 *  PCI Device ID Table
 *
 *  Used by probe to select devices to load on
 *  Last entry must be all 0s
 *
 *  { Vendor ID, Device ID, String }
 *********************************************************************/

#define PVID(vendor, devid, name) {vendor, devid, 0, 0, 0, 0, name}
#define PVID_OEM(vendor, devid, svid, sdevid, revid, name) {vendor, devid, svid, sdevid, revid, 0, name}
#define PVID_END {0, 0, 0, 0, 0, 0, NULL}

typedef struct pci_vendor_info {
	uint32_t	pvi_vendor_id;
	uint32_t	pvi_device_id;
	uint32_t	pvi_subvendor_id;
	uint32_t	pvi_subdevice_id;
	uint32_t	pvi_rev_id;
	uint32_t	pvi_class_mask;
	caddr_t		pvi_name;
} pci_vendor_info_t;

static pci_vendor_info_t igc_vendor_info_array[] =
{
	/* Intel(R) PRO/1000 Network Connection - igc */
	PVID(0x8086, IGC_DEV_ID_I225_LM, "Intel(R) Ethernet Controller I225-LM"),
	PVID(0x8086, IGC_DEV_ID_I225_V, "Intel(R) Ethernet Controller I225-V"),
	PVID(0x8086, IGC_DEV_ID_I225_K, "Intel(R) Ethernet Controller I225-K"),
	PVID(0x8086, IGC_DEV_ID_I225_I, "Intel(R) Ethernet Controller I225-I"),
	PVID(0x8086, IGC_DEV_ID_I220_V, "Intel(R) Ethernet Controller I220-V"),
	PVID(0x8086, IGC_DEV_ID_I225_K2, "Intel(R) Ethernet Controller I225-K(2)"),
	PVID(0x8086, IGC_DEV_ID_I225_LMVP, "Intel(R) Ethernet Controller I225-LMvP(2)"),
	PVID(0x8086, IGC_DEV_ID_I226_K, "Intel(R) Ethernet Controller I226-K"),
	PVID(0x8086, IGC_DEV_ID_I225_IT, "Intel(R) Ethernet Controller I225-IT(2)"),
	PVID(0x8086, IGC_DEV_ID_I226_LM, "Intel(R) Ethernet Controller I226-LM"),
	PVID(0x8086, IGC_DEV_ID_I226_V, "Intel(R) Ethernet Controller I226-V"),
	PVID(0x8086, IGC_DEV_ID_I226_IT, "Intel(R) Ethernet Controller I226-IT"),
	PVID(0x8086, IGC_DEV_ID_I221_V, "Intel(R) Ethernet Controller I221-V"),
	PVID(0x8086, IGC_DEV_ID_I226_BLANK_NVM, "Intel(R) Ethernet Controller I226(blankNVM)"),
	PVID(0x8086, IGC_DEV_ID_I225_BLANK_NVM, "Intel(R) Ethernet Controller I225(blankNVM)"),
	/* required last entry */
	PVID_END
};

#define INTEL_IGC_REGISTERMAP_SIZE 0x20000

#ifdef MULTI_QUEUE_SUPPORT
#define MAX_NUMBER_OF_TX_QUEUES 4
#define MAX_NUMBER_OF_RX_QUEUES	4
#define MAX_NUMBER_OF_QUEUES	4 // Max(MAX_NUMBER_OF_TX_QUEUES, MAX_NUMBER_OF_QUEUES)
#else
#define MAX_NUMBER_OF_TX_QUEUES 1	// change it to 4 
#define MAX_NUMBER_OF_RX_QUEUES	1
#define MAX_NUMBER_OF_QUEUES	1 // Max(MAX_NUMBER_OF_TX_QUEUES, MAX_NUMBER_OF_QUEUES)

#endif

// Ethernet values
#define ETHERNET_ADDRESS_LEN   6
#define ETHERNET_ETYPE_LEN     2
#define ETHERNET_HEADER_LEN    14
#define ETHERNET_FCS_LEN       4     // Checksum length
#define ETHERNET_DATA_LEN      1500  // Maximum (standard) data (w/o HDR/FCS)
#define ETHERNET_MIN_LEN       64    // Minumum frame
#define ETHERNET_STD_LEN       1518  // Maximum (standard) frame 

#define ETHERNET_ETYPE_OFFSET		12
#define ETHERNET_VLANTAG_OFFSET		12

// Maximum jumbo frame size (including Ethernet FCS) supported for RtNalIGC devices
#define DRV_MAX_JUMBO_LEN		(9 * 1024)

// Default RX buffer Size
#define DRV_RXBUFFER_2048  2048

// Number of Transmit Descriptors
#define DEFAULT_TXD      256 
#define MAX_TXD         16384
#define MIN_TXD          80 

// Number of Receive Descriptors
#define DEFAULT_RXD      256 
#define MAX_RXD         16384 
#define MIN_RXD          80 

#define REQ_TX_DESCRIPTOR_MULTIPLE 8
#define REQ_RX_DESCRIPTOR_MULTIPLE 8

#define	le32toh(x)	((uint32_t)(x))

typedef LARGE_INTEGER PHYSICAL_ADDRESS, * PPHYSICAL_ADDRESS;


#define DebugMsg(_Data_)  if (pGlobals->displayErrorMessages){ \
								RtPrintf("%s ",pGlobals->msgPrefix); \
								RtPrintf _Data_ ; } \
														else (void)0

#define ROUNDUP(size, unit) (unit * ((size + unit - 1) / unit))

#ifndef roundup2
#define	roundup2(x, y)	(((x)+((y)-1))&(~((y)-1))) /* if y is powers of two */
#endif

typedef struct _COMMON_DEV_STATS
{
	unsigned long Size;     // Size of structure
	unsigned long Sbrcnt;  // Receive packet count
	unsigned long Sbxcnt;  // Transmit packet count
	unsigned long Sbrecnt; // Receive error count
	unsigned long Sbxecnt; // Transmit error count
	unsigned long Sbjam;   // Jam retry count 
	unsigned long SbXnbc;	// Transmit no buffer count
} COMMON_DEV_STATS, * PCOMMON_DEV_STATS;


//-----------------------------------------------------------------------------
// Descriptor (transmit and receive) structures
//
typedef struct _IGC_ADV_TRANSMIT_DESCRIPTOR
{
	union
	{
		struct
		{
			volatile PHYSICAL_ADDRESS buffer_addr;		// BufferAddress; 
			volatile DWORD cmd_type_len;				// typeAndLength;			
			volatile DWORD olinfo_status;				// paylen;
		} read; // config;

		struct {
			volatile DWORD64 reserved;  // DmaTimeStamp;
			volatile DWORD nxtseq_seed; // reserved;
			volatile DWORD status;      // sta;
		} wb; // stats;
	};
} IGC_ADV_TRANSMIT_DESCRIPTOR, * TIGC_ADV_TRANSMIT_DESCRIPTOR;


/* Context descriptors */
typedef struct _IGC_ADV_TX_CONTEXT_DESCRIPTOR
{
	__int32 vlan_maclen_iplen;
	union {
		__le32 launch_time;
		__le32 seqnum_seed;
	}; //__int32 launchTime;
	__int32 type_tucmd_mlhl;
	__int32 mss_l4len_idx;
} IGC_ADV_TX_CONTEXT_DESCRIPTOR, * TIGC_ADV_TX_CONTEXT_DESCRIPTOR;



/* Receive Descriptor */
typedef struct _RTIGC_RECEIVE_DESCRIPTOR
{
	PHYSICAL_ADDRESS BufferAddress; /* Address of the descriptor's data buffer */
	USHORT Length; /* Length of data DMAed into data buffer */
	USHORT Csum; /* Packet checksum */
	UCHAR ReceiveStatus; /* Descriptor status */
	UCHAR Errors; /* Descriptor Errors */
	USHORT VlanTag;

} IGC_RECEIVE_DESCRIPTOR, * TIGC_RECEIVE_DESCRIPTOR;


typedef struct _IGC_EXT_RECEIVE_DESCRIPTOR
{
	/* Receive Descriptor - Extended */
	union {
		struct {
			PHYSICAL_ADDRESS buffer_addr;
			PHYSICAL_ADDRESS reserved;
		} read;
		struct { //wb
			struct { //lower
				__le32 mrq; /* Multiple Rx Queues */
				union {
					__le32 rss; /* RSS Hash */
					struct {
						__le16 ip_id;  /* IP id */
						__le16 csum;   /* Packet Checksum */
					} csum_ip;
				} hi_dword;
			} lower;
			struct {
				__le32 status_error;  /* ext status/error */
				__le16 length;
				__le16 vlan; /* VLAN tag */
			} upper;
		} wb;  /* writeback */
	};
}IGC_EXT_RECEIVE_DESCRIPTOR, * PIGC_EXT_RECEIVE_DESCRIPTOR;

typedef struct _IGC_ADV_RECEIVE_DESCRIPTOR
{
	/* Receive Descriptor - Advanced */
	union  {
		struct {
			PHYSICAL_ADDRESS pkt_addr; /* Packet buffer address */
			PHYSICAL_ADDRESS hdr_addr; /* Header buffer address */
		} read;
		struct {
			struct {
				union {
					__le32 data;
					struct {
						__le16 pkt_info; /*RSS type, Pkt type*/
						/* Split Header, header buffer len */
						__le16 hdr_info;
					} hs_rss;
				} lo_dword;
				union {
					__le32 rss; /* RSS Hash */
					struct {
						__le16 ip_id; /* IP id */
						__le16 csum; /* Packet Checksum */
					} csum_ip;
				} hi_dword;
			} lower;
			struct {
				__le32 status_error; /* ext status/error */
				__le16 length; /* Packet length */
				__le16 vlan; /* VLAN tag */
			} upper;
		} wb;  /* writeback */
	};
} IGC_ADV_RECEIVE_DESCRIPTOR, * PIGC_ADV_RECEIVE_DESCRIPTOR;

typedef RTNAL_HW_TIME PTP_TIME;
typedef RTNAL_HW_TIMESTAMPS PTP_HW_TSTAMPS;
typedef RTNAL_HW_QPC_TIMESTAMPS PTP_HW_QPC_TSTAMPS;


typedef struct _HFREECRITICALLOCK
{
	LONG volatile	lock;  // this variable must be first in this structure
	DWORD volatile	lockOwnerThreadID;  // acquired from GetCurrentThreadId(VOID);
	DWORD volatile	lockOwnerThreadPriority;
	DWORD volatile	lockRequestorThreadPriority;  // original priority before priority inversion 
	DWORD			maxNumberOfSpins;
	char			lockSymbolic[MAX_PATH];  // string describing what the lock is used for

} HFREECRITICALLOCK;

// Flags to track the status of the NAL application using the receive queue
#define DRV_RX_ATTACHED_FLAG	0x0001
#define DRV_RX_NOTIFY_FLAG		0x0002
#define DRV_RX_POLLING_FLAG		0x0004
#define DRV_RX_POLLING			(DRV_RX_ATTACHED_FLAG | DRV_RX_POLLING_FLAG)
#define DRV_RX_NOTIFY			(DRV_RX_ATTACHED_FLAG | DRV_RX_NOTIFY_FLAG)
#define DRV_RX_FLAGS			(DRV_RX_ATTACHED_FLAG | DRV_RX_POLLING_FLAG | DRV_RX_NOTIFY_FLAG)

#define DRV_IS_RX_POLLING(flags)	((flags & DRV_RX_POLLING) == DRV_RX_POLLING)
#define DRV_IS_RX_NOTIFY(flags)		((flags & DRV_RX_NOTIFY) == DRV_RX_NOTIFY)

#define DRV_IS_RX_ATTACHED(flags)	((flags & DRV_RX_ATTACHED_FLAG) != 0)

// Flags to track the status of the NAL application using the transmit queue
#define DRV_TX_ATTACHED_FLAG	0x1000
#define DRV_TX_NOTIFY_FLAG		0x2000
#define DRV_TX_NOTIFY			(DRV_TX_ATTACHED_FLAG | DRV_TX_NOTIFY_FLAG)
#define DRV_TX_FLAGS			(DRV_TX_ATTACHED_FLAG | DRV_TX_NOTIFY_FLAG)

#define DRV_IS_TX_NOTIFY(flags)		((flags & DRV_TX_NOTIFY) == DRV_TX_NOTIFY)

typedef struct _DRV_QUE_DESCRIPTORS
{
	//  Allocated Memory blocks prior to alignment
	BYTE* pTxDescriptorBufferUnAligned;
	BYTE* pRxDescriptorBufferUnAligned;

	// Aligned memory blocks
	BYTE* pbyTxDescriptors;
	BYTE* pbyRxDescriptors;

	// Transmit descriptor variables
	IGC_ADV_TRANSMIT_DESCRIPTOR* TxDescriptors;
	PRTNAL_FRAME* pTxNalFrames;					// Array of pointers to NAL frames
	BYTE**	txFrameBuffVirtualAddr;				// Points to an array of virtual addresses for frame buffers, allocated by the driver
	PHYSICAL_ADDRESS* txFrameBuffPhysicalAddr;	// Points to an array of physical addresses for frame buffers, allocated by the driver
	PHYSICAL_ADDRESS TxDescDMA;
	DWORD			TxIntDelay;
	DWORD			TxAbsIntDelay;
	DWORD			NumTxDescriptors;
	volatile DWORD	NextAvailTxDescriptor;
	volatile DWORD	OldestUsedTxDescriptor;
	volatile DWORD	NumTxDescriptorsAvail;

	// Receive descriptor variables
	IGC_ADV_RECEIVE_DESCRIPTOR* RxDescriptors;

	RTNAL_FRAME* RxNalFrames;					// Array of NAL frames
	BYTE** rxFrameBuffVirtualAddr;				// Points to an array of virtual addresses for frame buffers, allocated by the driver
	PHYSICAL_ADDRESS* rxFrameBuffPhysicalAddr;	// Points to an array of physical addresses for frame buffers, allocated by the driver
	PHYSICAL_ADDRESS RxDescDMA;
	DWORD			RxIntDelay;
	DWORD	RxAbsIntDelay;
	volatile DWORD	NumRxDescriptors;
	volatile DWORD	NextRxDescriptorToCheck;
	volatile DWORD	NextRxDescriptorToFill;
	volatile ULONG  RxDescriptorsNotified;

	volatile void* pRxAppContext;
	volatile DWORD	flags;

	//BOOL      bTimeStamp; // Enable/Disable time stamping on queue
	BOOL      bRxMonitor; // Enable/disable monitor events for receive
	BOOL      bTxMonitor; // Enable/disable monitor events for transmit

}  DRV_QUE_DESCRIPTORS, * PDRV_QUE_DESCRIPTORS;

// Kind of assetted interrupt
typedef enum _INTERRUPT_KIND
{
	InterruptKindNone = 0,
	InterruptKindTransmit = 1,
	InterruptKindReceive = 2,
	InterruptKindLineStatusChange = 3
} INTERRUPT_KIND;

// Asserted interrupt and queue it belongs to 
typedef struct _INTERRUPT_AND_QUEUE
{
	ULONG queue;
	INTERRUPT_KIND kind;
} INTERRUPT_AND_QUEUE;

// This structure defines a mapping of an MSIX multiple vector message to queues and message types that it's configured to be asserted for.  
typedef struct _MSIXMV_MESSAGE_QUEUES
{
	ULONG messageId;
	ULONG processor;
	// Maximum amount of 
	INTERRUPT_AND_QUEUE queues[MAX_NUMBER_OF_QUEUES + 1];
	ULONG numberOfQueues;
} MSIXMV_MESSAGE_QUEUES;

typedef struct _DRVDEV
{
	//
	// These are required by the stack interface
	//
	void* ndp;
	char StackDeviceName[RTN_MAX_DEVICE_NAME];

	ULONG_PTR IntAffinityMask;

	//
	// These are device-specific, but are very common and will probably be required
	//
	ULONG Irq; // IRQ number
	ULONG BusNumber;
	ULONG FuncNumber;
	PCI_SLOT_NUMBER SlotNumber;

	BOOL LineBasedOnly;
	BOOL UsingMSI;
	BOOL PreferMsi;
	ULONG IntIdealProc;
	ULONG IntPriority;
	BOOL  LinkStatus;
	ULONG LineStatusPriority;
	ULONG LineStatusIdealProcessor;
	BOOL RunTxCompleteThread;
	ULONG TxCompletePriority;
	ULONG TxCompleteIdealProcessor;
	
	HANDLE hInterrupt;
	HANDLE hTxCompleteEvent;
	HANDLE hLineStatusEvent;
	HANDLE hLineStatusThread;
	HANDLE hDrvTxCompleteThread;
	
	HFREECRITICALLOCK csRxDMARing[MAX_NUMBER_OF_RX_QUEUES];
	HFREECRITICALLOCK csTxDMARing[MAX_NUMBER_OF_TX_QUEUES];
	HFREECRITICALLOCK csIoctl;

	ULONG ulLatencyRecvUpdated;

	ULONG IOBase;     // IO space base address
	UCHAR* CsrBase;   // CSR memory base address of registers on the card
	PHYSICAL_ADDRESS CsrPhysical;

	// Has the device been brought up?
	BOOL DeviceIsUp;

	// Common network device statistics
	COMMON_DEV_STATS ComStats;

	// Mac and Phy Settings
#ifdef TIMESTAMP_SUPPORT
	DWORD	PhyTxDelay;
	DWORD	PhyRxDelay;
#endif

	// Autoneg configuration 
	BOOL 	AutoNeg;
	BOOL 	AutoNegAdv;
	BOOL 	WaitAutoNegComplete;
	BOOL 	ForcedSpeedDuplex;
	BOOL 	SpeedDowngraded;
	USHORT  AutoNegAdvertised;
	BOOL    AutoNegFailed;

	// Status Flags
	RTND_MEDIA_CONNECT_STATE	LinkIsActive;
	USHORT		LineSpeed;
	USHORT		FullDuplex;
	BOOL		GetLinkStatus;
	BOOL		TbiCompatibilityEnable;
	BOOL		TbiCompatibilityOn;

	// PCI Device Info
	USHORT		VendorId;
	USHORT		DeviceId;
	BYTE		RevId;
	DWORD		adapterType;

	//  Queue info
	int			numberOfRxNicQueues;
	int			numberOfTxNicQueues;
	int			defaultQueue;

	// For normal Rx/Tx usage, the maximum number of messages cannot exceed twice the number of queues
	MSIXMV_MESSAGE_QUEUES messageToInerruptKindQueueMap[MAX_NUMBER_OF_QUEUES * 2 + 1];
	ULONG usedMessagesCount;

	// Change McastFlags to 64 bit if number of RX queues is > 32
	DWORD		McastFlags;
	DWORD		OriginalFlowControl;
	DWORD		FlowControl;

	DRV_QUE_DESCRIPTORS	adapterQ[MAX_NUMBER_OF_QUEUES];

	// Frame/MTU values
	ULONG		Mtu;
	ULONG		MaxPacket;
	ULONG		RxBufferSize;

	// TO settings
	BYTE RxChecksum;

	// Our Ethernet address
	BYTE EnetAddr[6]; // Physical address: from TCP/IP or

	// Statistic Registers Dump
	__int64 Crcerrs;         // rx crc error count
	__int64 Algnerrc;        // alignment error count (LG)
	__int64 Symerrs;         // rx symbol error count
	__int64 Rxerrc;          // RX_ERROR count (LG) - PHY detected rx errors
	__int64 Mpc;             // Missed Packet Count - packets dropped due to full rx fifo
	__int64 Scc;             // single collision count
	__int64 Ecol;            // excessive collision count
	__int64 Mcc;             // multiple collision count
	__int64 Latecol;         // late collision count
	__int64 Colc;            // Total collision count
	__int64 Tuc;             // transmit underrun count (LG)
	__int64 Dc;              // tx defer count due
	__int64 Tncrs;           // transmit no CRS count (LG)
	__int64 Sec;             // rx sequence error count
	__int64 Cexterr;         // carrier ext error count (LG)
	__int64 Rlec;            // receive length error count
	__int64 Rutec;           // receive DMA to early count (LG)
	__int64 Xonrxc;          // XON received count
	__int64 Xontxc;          // XON transmitted count
	__int64 Xoffrxc;         // XOFF received count
	__int64 Xofftxc;         // XOFF transmitted count
	__int64 Fcruc;           // unknown Flow Control packet rcved count
	__int64 Prc64;           // packets received (64 Bytes)
	__int64 Prc127;          // packets received (65-127)
	__int64 Prc255;          // packets received (128-255)
	__int64 Prc511;          // packets received (256-511)
	__int64 Prc1023;         // packets received (512-1023)
	__int64 Prc1522;         // packets received (1024+)
	__int64 Gprc;            // Good packets received count
	__int64 Bprc;            // Broadcasts pkts recvd count
	__int64 Mprc;            // Multicast  pkts recvd count
	__int64 Gptc;            // Good packets xmited count
	__int64 Gor;             // Good octets recvd
	__int64 Got;             // Good octets xmitd
	__int64 Rnbc;            // Receive no buffers count
	__int64 Ruc;             // Receive undersize count
	__int64 Rfc;             // Receive fragment count
	__int64 Roc;             // Receive oversize count
	__int64 Rjc;             // Receive jabber count
	__int64 Tor;             // Total octets recvd
	__int64 Tot;             // Total octets xmitd
	__int64 Tpr;             // Total packets received
	__int64 Tpt;             // Total packets xmitted
	__int64 Ptc64;           // packets xmitted (64B)
	__int64 Ptc127;          // packets xmitted (64-127)
	__int64 Ptc255;          // packets xmitted (128-255)
	__int64 Ptc511;          // packets xmitted (256-511)
	__int64 Ptc1023;         // packets xmitted (512-1023)
	__int64 Ptc1522;         // packets xmitted (1024-1522)
	__int64 Mptc;            // Multicast  pkts xmit count
	__int64 Bptc;            // Broadcasts pkts xmit count


	// Type of onboard NVM
	//NVM_TYPE NvmType;

	BOOL ZeroCopy;
	
	struct igc_adapter adapter;

} DRVDEV, * PDRVDEV;




#define DRV_MAX_PORTS 64

typedef struct _DRV_GLOBALS
{
	ULONG		nCards;
	DRVDEV* drvCards[DRV_MAX_PORTS];
	char* msgPrefix;
	BOOL		displayErrorMessages;
	PTP_HW_TSTAMPS txTimeStamp, rxTimeStamp;
	DWORD		reserved[256];
} DRV_GLOBALS;

DRV_GLOBALS g_DrvGlobals;
DRV_GLOBALS* pGlobals;

// This structure is used to separate the initialization (input) parameters from
// the actual card values.
typedef struct _DRVDEV_INIT_PARAMS
{
	BYTE MacAddress[6];
	ULONG BusNumber;
	ULONG DeviceNumber;
	ULONG FuncNumber;
	DRVDEV* pCardInfo;
} DRVDEV_INIT_PARAMS, * PDRVDEV_INIT_PARAMS;

