/******************************************************************************

  Copyright (c) 2001-2015, Intel Corporation 
  All rights reserved.
  
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:
  
   1. Redistributions of source code must retain the above copyright notice, 
      this list of conditions and the following disclaimer.
  
   2. Redistributions in binary form must reproduce the above copyright 
      notice, this list of conditions and the following disclaimer in the 
      documentation and/or other materials provided with the distribution.
  
   3. Neither the name of the Intel Corporation nor the names of its 
      contributors may be used to endorse or promote products derived from 
      this software without specific prior written permission.
  
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.

******************************************************************************/
/*$FreeBSD$*/


#include "RtNalIGC.h"
#include "igc_osdep.h"
#include "pcireg.h"

// RTX_PORT
u32 pci_read_config(DRVDEV *pCardInfo, u32 offset, u32 len)
{
	u32 retValue = 0;

	if (len <= sizeof(u32))
	{
		RtGetBusDataByOffset(PCIConfiguration,
			pCardInfo->BusNumber,
			pCardInfo->SlotNumber.u.AsULONG,
			(PVOID)&retValue,
			offset,
			len
			);
	}

	return(retValue);
}


void pci_write_config(DRVDEV *pCardInfo, u32 offset, u32 data, u32 len)
{

	if (len <= sizeof(u32))
	{
		RtSetBusDataByOffset(PCIConfiguration,
			pCardInfo->BusNumber,
			pCardInfo->SlotNumber.u.AsULONG,
			(PVOID)&data,
			offset,
			len
			);
	}

	return;

}







/*
* Find the requested capability and return the offset in
* configuration space via the pointer provided.  The function returns
* 0 on success and an error code otherwise.
*/
int
pci_find_cap(DRVDEV *pCardInfo, int capability, int *capreg)
{
	//struct pci_devinfo *dinfo = device_get_ivars(child);
	//pcicfgregs *cfg = &dinfo->cfg;
	DWORD status;
	unsigned __int8 ptr;


	

	/*
	* Check the CAP_LIST bit of the PCI status register first.
	*/
	status = pci_read_config(pCardInfo, PCIR_STATUS, 2);//     pci_read_config(child, PCIR_STATUS, 2);
	if (!(status & PCIM_STATUS_CAPPRESENT))
		return (ENXIO);

	/*
	* Determine the start pointer of the capabilities list.
	*/
	//switch (cfg->hdrtype & PCIM_HDRTYPE) {
	//case PCIM_HDRTYPE_NORMAL:
	//case PCIM_HDRTYPE_BRIDGE:
	//	ptr = PCIR_CAP_PTR;
	//	break;
	//case PCIM_HDRTYPE_CARDBUS:
	//	ptr = PCIR_CAP_PTR_2;
	//	break;
	//default:
	//	/* XXX: panic? */
	//	return (ENXIO);		/* no extended capabilities support */
	//}
	ptr = PCIR_CAP_PTR;
	ptr = pci_read_config(pCardInfo, ptr, 1);

	/*
	* Traverse the capabilities list.
	*/
	while (ptr != 0) {
		if (pci_read_config(pCardInfo, ptr + PCICAP_ID, 1) == capability) {
			if (capreg != NULL)
				*capreg = ptr;
			return (0);
		}
		ptr = pci_read_config(pCardInfo, ptr + PCICAP_NEXTPTR, 1);
	}

	return (-1);
}



const char* device_get_name(DRVDEV *dev)
{
	if (dev && dev->StackDeviceName[0])
		return (dev->StackDeviceName);

	return (NULL);
}

const char*
device_get_nameunit(DRVDEV *dev)
{
	return device_get_name(dev);
}

struct resource* bus_alloc_resource_any(device_t dev, int type, int* rid, unsigned int flags)
{
	struct resource* res;
	DRVDEV* pCardInfo = (DRVDEV*)dev;
	static u64 bustag;
	bus_space_handle_t bus_handle = 0;

	if (dev == NULL || rid == NULL)
		return NULL;

	if (type == SYS_RES_MEMORY) {
		bus_handle = (bus_space_handle_t)pCardInfo->CsrBase;
	}
	else if (type == SYS_RES_IOPORT)
	{
		bus_handle = (bus_space_handle_t)pCardInfo->IOBase;
	}

	if (bus_handle == 0)
		return NULL;

	res = malloc(sizeof * res);
	if (res == NULL)
		return (NULL);

	res->r_spec.type = type;
	res->r_spec.rid = *rid;
	res->r_spec.flags = flags;

	res->r_bustag = ++bustag;
	res->r_bushandle = bus_handle;

	return res;
}

int bus_release_resource(device_t dev, int type, int rid, struct resource* res)
{
	int error = ENOENT;
	if (res && res->r_spec.type == type && res->r_spec.rid == rid)
	{
		free(res);
		error = 0;
	}
	return error;
}