/******************************************************************************

  Copyright (c) 2001-2015, Intel Corporation
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

   1. Redistributions of source code must retain the above copyright notice,
	  this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
	  notice, this list of conditions and the following disclaimer in the
	  documentation and/or other materials provided with the distribution.

   3. Neither the name of the Intel Corporation nor the names of its
	  contributors may be used to endorse or promote products derived from
	  this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.

******************************************************************************/
/*$FreeBSD$*/


#ifndef _IGC_OS_H_
#define _IGC_OS_H_

// RTX_PORT_START

#include <Windows.h>
#include <rtapi.h>      // RTX64 APIs that can be used in real-time or Windows applications.

#include "rtnapi.h"
#include <RtNalApi.h>
#ifdef UNDER_RTSS
#include <rtssapi.h>    // RTX64 APIs that can only be used in real-time applications.
#endif // UNDER_RTSS


#ifdef UNDER_RTSS
#undef printf
#define printf	RtPrintf
#undef wprintf
#define wprintf RtWprintf
#endif
//#endif

#define ASSERT(x) if(!(x)) RtPrintf("Panic IGC: x")
#define EWARN(H, W)  RtPrintf(W)

#define MyHtonl(x) (u32)((u32)(((x) >> 24)& 0xff)\
	| (u32)(((x) >> 8)&0xff00)\
	| (u32)(((x & 0xffffff) << 8)& 0xff0000)\
	| (u32)(((x & 0xff) << 24)&0xff000000))

#define MyHtons(x) (u16)((u16)(((x)>>8)& 0xffU) \
              |(u16)(((x)& 0xffU)<<8))

#define IGC_CPU_TO_LE16(x) (u16)(x)

#define IGC_BE32_TO_CPU(x) (u32)MyHtonl((u32)(x))

#ifndef htonl
#define htonl MyHtonl
#endif
#ifndef nhtol
#define nhtol MyHtonl
#endif
#ifndef htons
#define htons MyHtons
#endif
#ifndef nhtos
#define nhtos MyHtons
#endif

#if(UNDER_RTSS)

__inline void DELAY(int microseconds)
{
	static LARGE_INTEGER freq = { 0, 0 };
	LARGE_INTEGER start;
	LARGE_INTEGER delay;
	LARGE_INTEGER now;
	volatile int cycles = 0;

	if (freq.QuadPart == 0)
	{
		QueryPerformanceFrequency(&freq);
	}

	delay.QuadPart = microseconds * freq.QuadPart / 1000000;

	QueryPerformanceCounter(&start);
	do
	{
		QueryPerformanceCounter(&now);
		// do busy wait here
		++cycles;
	} while (now.QuadPart - start.QuadPart < delay.QuadPart);
}

/* The happy-fun DELAY macro is defined in /usr/src/sys/i386/include/clock.h */
#define usec_delay(x) DELAY(x)
#define usec_delay_irq(x) usec_delay(x)
#define msec_delay(x) DELAY(1000*(x))
#define msec_delay_irq(x) DELAY(1000*(x))

#define IGC_ERROR_UNSUPPORTED "IGC_ERROR_UNSUPPORTED: "
#define IGC_ERROR_ARGUMENT	"IGC_ERROR_ARGUMENT: "
#define IGC_ERROR_POLLING		"IGC_ERROR_POLLING: "
#define IGC_ERROR_SOFTWARE	"IGC_ERROR_SOFTWARE: "
#define IGC_ERROR_CAUTION		"IGC_ERROR_CAUTION: "
#define IGC_ERROR_INVALID_STATE "IGC_ERROR_INVALID_STATE: "

#define DBG 0
#define MSGOUT(S, A, B)     printf(S "\n", A, B)
#if DBG
	#define DEBUGOUT(S)         printf(S "\n")
	#define DEBUGOUT1(S,A)      printf(S "\n",A)
	#define DEBUGOUT2(S,A,B)    printf(S "\n",A,B)
	#define DEBUGOUT3(S,A,B,C)  printf(S "\n",A,B,C)
	#define DEBUGOUT4(S,A,B,C,D)  printf(S "\n",A,B,C,D)
	#define DEBUGOUT5(S,A,B,C,D,E)  printf(S "\n",A,B,C,D,E)
	#define DEBUGOUT6(S,A,B,C,D,E,F)  printf(S "\n",A,B,C,D,E,F)
	#define DEBUGOUT7(S,A,B,C,D,E,F,G)  printf(S "\n",A,B,C,D,E,F,G)
	#define ERROR_REPORT1(S,A)      printf(S A "\n")
	#define ERROR_REPORT2(S,A,B)    printf(S A "\n",B)
	#define ERROR_REPORT3(S,A,B,C)  printf(S A "\n",B,C)
#else
	#define DEBUGOUT(S)
	#define DEBUGOUT1(S,A)
	#define DEBUGOUT2(S,A,B)
	#define DEBUGOUT3(S,A,B,C)
	#define DEBUGOUT4(S,A,B,C,D)
	#define DEBUGOUT5(S,A,B,C,D,E)
	#define DEBUGOUT6(S,A,B,C,D,E,F)
	#define DEBUGOUT7(S,A,B,C,D,E,F,G)

	#define ERROR_REPORT1(S,A)
	#define ERROR_REPORT2(S,A,B)
	#define ERROR_REPORT3(S,A,B,C)
#endif

#define DEBUGFUNC(F)        DEBUGOUT(F);

/* Defines for printing debug information */
#define DEBUG_INIT  0
#define DEBUG_IOCTL 0
#define DEBUG_HW    0

#define INIT_DEBUGOUT(S)            if (DEBUG_INIT)  printf(S "\n")
#define INIT_DEBUGOUT1(S, A)        if (DEBUG_INIT)  printf(S "\n", A)
#define INIT_DEBUGOUT2(S, A, B)     if (DEBUG_INIT)  printf(S "\n", A, B)
#define IOCTL_DEBUGOUT(S)           if (DEBUG_IOCTL) printf(S "\n")
#define IOCTL_DEBUGOUT1(S, A)       if (DEBUG_IOCTL) printf(S "\n", A)
#define IOCTL_DEBUGOUT2(S, A, B)    if (DEBUG_IOCTL) printf(S "\n", A, B)
#define HW_DEBUGOUT(S)              if (DEBUG_HW) printf(S "\n")
#define HW_DEBUGOUT1(S, A)          if (DEBUG_HW) printf(S "\n", A)
#define HW_DEBUGOUT2(S, A, B)       if (DEBUG_HW) printf(S "\n", A, B)


// RTX_PORT_END

#define FALSE               0
#define false               0 /* shared code requires this */
#define TRUE                1
#define true                1
#define CMD_MEM_WRT_INVALIDATE          0x0010  /* BIT_4 */
#define PCI_COMMAND_REGISTER            PCIR_COMMAND

/* Shared code dropped this define.. */
#define IGC_INTEL_VENDOR_ID		0x8086

/* Bunch of defines for shared code bogosity */
//#define UNREFERENCED_PARAMETER(_p)   //  RTX_PORT_START  RTX_PORT_END
#define UNREFERENCED_1PARAMETER(_p)
#define UNREFERENCED_2PARAMETER(_p, _q)
#define UNREFERENCED_3PARAMETER(_p, _q, _r)
#define UNREFERENCED_4PARAMETER(_p, _q, _r, _s)


#ifdef UNDER_RTSS  // RTX_PORT_START  RTX_PORT_END
typedef unsigned __int8		uint8_t	;
typedef __int8				int8_t;
typedef unsigned __int16	uint16_t;
typedef __int16				int16_t;
typedef unsigned __int32	uint32_t;
typedef __int32				int32_t;
typedef unsigned __int64	uint64_t;
typedef __int64				int64_t;
typedef unsigned __int32	boolean_t;

#define IGC_NTOHL(_i)	nhtol(_i)
#define IGC_NTOHS(_i)	nhtos(_i)

/* XXX these need to be revisited */
#define IGC_CPU_TO_LE32(x) x			// this was htole32 but since Intel is alread little endian do nothing RTX_PORT_START  RTX_PORT_END
#define IGC_LE32_TO_CPUS(x)
#define IGC_CPU_TO_BE16 htons		// - this was  htobe16  RTX_PORT_START  RTX_PORT_END
#define IGC_CPU_TO_BE32 htonl		//  - this was  htobe32  RTX_PORT_START  RTX_PORT_END

#endif // UNDER_RTSS  RTX_PORT_START  RTX_PORT_END

typedef uint8_t		u8;
typedef int8_t		s8;
typedef uint16_t	u16;
typedef int16_t		s16;
typedef uint32_t	u32;
typedef int32_t		s32;
typedef uint64_t	u64;
#ifndef __bool_true_false_are_defined
typedef boolean_t	bool;
#endif

/* shared code requires this */
#define __le16  u16
#define __le32  u32
#define __le64  u64
#define __be16  u16
#define __be32  u32
#define __be64  u64

#define le16_to_cpu 

typedef void* caddr_t;
#define  device_t void *
typedef uint16_t qidx_t;

#if __FreeBSD_version < 800000
#if defined(__i386__) || defined(__amd64__)
#define mb()	__asm volatile("mfence" ::: "memory")
#define wmb()	__asm volatile("sfence" ::: "memory")
#define rmb()	__asm volatile("lfence" ::: "memory")
#else
#define mb()
#define rmb()
#define wmb()
#endif
#endif

#if defined(__i386__) || defined(__amd64__)
static __inline
void prefetch(void *x)
{
	__asm volatile("prefetcht0 %0" :: "m" (*(unsigned long *)x));
}
#else
#define prefetch(x)
#endif


#if(!UNDER_RTSS) // RTX_PORT_START  

struct igc_osdep
{
	bus_space_tag_t    mem_bus_space_tag;
	bus_space_handle_t mem_bus_space_handle;
	struct device     *dev;
};

/* These routines are needed by the shared code */
struct igc_hw; 
extern u16 igc_read_pci_cfg(struct igc_hw *, u32);
#define IGC_READ_PCIE_WORD igc_read_pci_cfg

extern void igc_write_pci_cfg(struct igc_hw *, u32, u16);
#define IGC_WRITE_PCIE_WORD igc_write_pci_cfg

static __inline uint32_t
igc_read_reg_osdep(struct igc_osdep *osdep, uint32_t reg)
{
	uint32_t value;
	int count = 0;

	do {
		value = bus_space_read_4(osdep->mem_bus_space_tag,
		    osdep->mem_bus_space_handle, reg);
	} while (value == 0xdeadbeef && ++count < 10);

	if (count > 1)
		device_printf(osdep->dev, "%d register reads @ 0x%8x\n",
		    count, reg);

	return value;
}

#define IGC_READ_REG(a, reg) igc_read_reg_osdep((a)->back, reg)

#define IGC_WRITE_FLUSH(a) IGC_READ_REG(a, IGC_STATUS)

#define IGC_WRITE_REG(a, reg, value) (\
   bus_space_write_4( ((struct igc_osdep *)(a)->back)->mem_bus_space_tag, \
                     ((struct igc_osdep *)(a)->back)->mem_bus_space_handle, \
                     reg, value))


#define IGC_READ_REG_ARRAY(a, reg, offset) (\
   bus_space_read_4( ((struct igc_osdep *)(a)->back)->mem_bus_space_tag, \
                     ((struct igc_osdep *)(a)->back)->mem_bus_space_handle, \
                     (reg + ((offset) << 2))))

#define IGC_WRITE_REG_ARRAY(a, reg, offset, value) (\
      bus_space_write_4( ((struct igc_osdep *)(a)->back)->mem_bus_space_tag, \
                      ((struct igc_osdep *)(a)->back)->mem_bus_space_handle, \
                      (reg + ((offset) << 2)), value))


#else  

bool displayErrorMessages;
#define bootverbose displayErrorMessages

const char* device_get_nameunit(device_t dev);
#define	device_printf(dev, fmt,...) \
	if (bootverbose) printf("%s: " fmt, device_get_nameunit(dev),## __VA_ARGS__)

#define READ32(reg) (*((volatile DWORD *)(reg)))
#define WRITE32(reg,val) (*((volatile DWORD *)(reg))=(val))

#define IGC_READ_REG(a, reg) (READ32((a)->hw_addr + (reg)))

#define IGC_WRITE_FLUSH(a) IGC_READ_REG(a, IGC_STATUS)
//#define IGC_WRITE_FLUSH(a) (READ32((a)->hw_addr + (IGC_STATUS)))

#define IGC_WRITE_REG(a, reg, value) (WRITE32((a)->hw_addr + (reg), value))	


#define IGC_READ_REG_ARRAY(a, reg, offset)  IGC_READ_REG(a, (reg) + ((offset) << 2))

#define IGC_WRITE_REG_ARRAY(a, reg, offset, value) IGC_WRITE_REG(a, (reg) + ((offset) << 2), value)

// from  "if_var.h"
//typedef enum {
//	IFCOUNTER_IPACKETS = 0,
//	IFCOUNTER_IERRORS,
//	IFCOUNTER_OPACKETS,
//	IFCOUNTER_OERRORS,
//	IFCOUNTER_COLLISIONS,
//	IFCOUNTER_IBYTES,
//	IFCOUNTER_OBYTES,
//	IFCOUNTER_IMCASTS,
//	IFCOUNTER_OMCASTS,
//	IFCOUNTER_IQDROPS,
//	IFCOUNTER_OQDROPS,
//	IFCOUNTER_NOPROTO,
//	IFCOUNTERS /* Array size. */
//} ift_counter;
//
//#define IFNAMSIZ 64

//struct ifmediareq {
//	char	ifm_name[IFNAMSIZ];	/* if name, e.g. "en0" */
//	int	ifm_current;		/* current media options */
//	int	ifm_mask;		/* don't care mask */
//	int	ifm_status;		/* media status */
//	int	ifm_active;		/* active options */
//	int	ifm_count;		/* # entries in ifm_ulist array */
//	int* ifm_ulist;		/* media words */
//};


/*
 * Shared options
 */
#define	IFM_FDX		0x00100000	/* Force full duplex */
#define	IFM_HDX		0x00200000	/* Force half duplex */
#define	IFM_FLOW	0x00400000	/* enable hardware flow control */
#define	IFM_FLAG0	0x01000000	/* Driver defined flag */
#define	IFM_FLAG1	0x02000000	/* Driver defined flag */
#define	IFM_FLAG2	0x04000000	/* Driver defined flag */
#define	IFM_LOOP	0x08000000	/* Put hardware in loopback */

/*
 * Masks
 */
#define	IFM_NMASK	0x000000e0	/* Network type */
#define	IFM_TMASK	0x0000001f	/* Media sub-type */
#define	IFM_IMASK	0xf0000000	/* Instance */
#define	IFM_ISHIFT	28		/* Instance shift */
#define	IFM_OMASK	0x0000ff00	/* Type specific options */
#define	IFM_MMASK	0x00070000	/* Mode */
#define	IFM_MSHIFT	16		/* Mode shift */
#define	IFM_GMASK	0x0ff00000	/* Global options */

 /*
  * Please update ieee8023ad_lacp.c:lacp_compose_key()
  * after adding new Ethernet media types.
  */
  /* Note IFM_X(511) is the max! */

  /* Ethernet option values; includes bits used for extended variant field */
#define	IFM_ETH_MASTER	0x00000100	/* master mode (1000baseT) */
#define	IFM_ETH_RXPAUSE	0x00000200	/* receive PAUSE frames */
#define	IFM_ETH_TXPAUSE	0x00000400	/* transmit PAUSE frames */
#define	IFM_ETH_XTYPE	0x00007800	/* extended media variants */
#define	IFM_ETH_XSHIFT	6		/* shift XTYPE next to TMASK */

/*
 * Status bits
 */
#define	IFM_AVALID	0x00000001	/* Active bit valid */
#define	IFM_ACTIVE	0x00000002	/* Interface attached to working net */

 /*
  * if_media Options word:
  *	Bits	Use
  *	----	-------
  *	0-4	Media variant
  *	5-7	Media type
  *	8-15	Type specific options (includes added variant bits on Ethernet)
  *	16-18	Mode (for multi-mode devices)
  *	19	RFU
  *	20-27	Shared (global) options
  *	28-31	Instance
  */

  /*
   * Ethernet
   * In order to use more than 31 subtypes, Ethernet uses some of the option
   * bits as part of the subtype field.  See the options section below for
   * relevant definitions
   */
#define	IFM_ETHER	0x00000020
#define	IFM_ETHER_SUBTYPE(x) (((x) & IFM_TMASK) | \
	(((x) & (IFM_ETH_XTYPE >> IFM_ETH_XSHIFT)) << IFM_ETH_XSHIFT))
#define	IFM_X(x) IFM_ETHER_SUBTYPE(x)	/* internal shorthand */
#define	IFM_ETHER_SUBTYPE_SET(x) (IFM_ETHER_SUBTYPE(x) | IFM_ETHER)
#define	IFM_ETHER_SUBTYPE_GET(x) ((x) & (IFM_TMASK|IFM_ETH_XTYPE))
#define	IFM_ETHER_IS_EXTENDED(x)	((x) & IFM_ETH_XTYPE)

#define	IFM_10_T	3		/* 10BaseT - RJ45 */
#define	IFM_100_TX	6		/* 100BaseTX - RJ45 */
#define	IFM_1000_T	16		/* 1000baseT - 4 pair cat 5 */
#define	IFM_2500_T	IFM_X(37)	/* 2500Base-T - RJ45 (NBaseT) */
   
/*
* Shared media sub-types
*/

#define	IFM_AUTO	0		/* Autoselect best media */
#define	IFM_MANUAL	1		/* Jumper/dipswitch selects media */
#define	IFM_NONE	2		/* Deselect all media */
/**
 * @brief Driver interrupt filter return values
 *
 * If a driver provides an interrupt filter routine it must return an
 * integer consisting of oring together zero or more of the following
 * flags:
 *
 *	FILTER_STRAY	- this device did not trigger the interrupt
 *	FILTER_HANDLED	- the interrupt has been fully handled and can be EOId
 *	FILTER_SCHEDULE_THREAD - the threaded interrupt handler should be
 *			  scheduled to execute
 *
 * If the driver does not provide a filter, then the interrupt code will
 * act is if the filter had returned FILTER_SCHEDULE_THREAD.  Note that it
 * is illegal to specify any other flag with FILTER_STRAY and that it is
 * illegal to not specify either of FILTER_HANDLED or FILTER_SCHEDULE_THREAD
 * if FILTER_STRAY is not specified.
 */
#define	FILTER_STRAY		0x01
#define	FILTER_HANDLED		0x02
#define	FILTER_SCHEDULE_THREAD	0x04

#define	IFF_PROMISC	0x100		/* (n) receive all packets */
#define	IFF_ALLMULTI	0x200		/* (n) receive all multicast packets */


 /*
  * Some basic Ethernet constants.
  */
#define	ETHER_ADDR_LEN		6	/* length of an Ethernet address */
#define	ETHER_TYPE_LEN		2	/* length of the Ethernet type field */
#define	ETHER_CRC_LEN		4	/* length of the Ethernet CRC */
#define ETHERMTU	       1500
#define	ETHER_HDR_LEN		(ETHER_ADDR_LEN*2+ETHER_TYPE_LEN)
#define	ETHER_MIN_LEN		64	/* minimum frame len, including CRC */
#define	ETHER_MAX_LEN		1518	/* maximum frame len, including CRC */

#define	ETHERTYPE_VLAN		0x8100	/* IEEE 802.1Q VLAN tagging (XXX conflicts) */

  /*
   * Values for if_link_state.
   */
#define	LINK_STATE_UNKNOWN	0	/* link invalid/unknown */
#define	LINK_STATE_DOWN		1	/* link is down */
#define	LINK_STATE_UP		2	/* link is up */


#define	SYS_RES_IRQ	1	/* interrupt lines */
#define	SYS_RES_DRQ	2	/* isa dma lines */
#define	SYS_RES_MEMORY	3	/* i/o memory */
#define	SYS_RES_IOPORT	4	/* i/o ports */

#define	RF_ALLOCATED	0x0001	/* resource has been reserved */
#define	RF_ACTIVE	0x0002	/* resource allocation has been activated */
#define	RF_SHAREABLE	0x0004	/* resource permits contemporaneous sharing */
#define	RF_SPARE1	0x0008
#define	RF_SPARE2	0x0010
#define	RF_FIRSTSHARE	0x0020	/* first in sharing list */
#define	RF_PREFETCHABLE	0x0040	/* resource is prefetchable */
#define	RF_OPTIONAL	0x0080	/* for bus_alloc_resources() */

typedef enum {
	IFLIB_INTR_LEGACY,
	IFLIB_INTR_MSI,
	IFLIB_INTR_MSIX
} iflib_intr_mode_t;

#include "if_bus.h"

struct resource_spec {
	int	type;
	int	rid;
	int	flags;
};

struct resource {
	//bus_space_tag_t		r_bustag; /* bus_space tag */
	//bus_space_handle_t	r_bushandle;	/* bus_space handle */
	uint64_t r_bustag;
	uint64_t r_bushandle;
	struct resource_spec r_spec;
};

struct igc_osdep
{
	bus_space_tag_t    mem_bus_space_tag;
	bus_space_handle_t mem_bus_space_handle;
	struct device* dev;
};

#endif // !UNDER_RTSS  // RTX_PORT_END



struct resource* bus_alloc_resource_any(device_t, int, int*, unsigned int);
int bus_release_resource(device_t, int, int, struct resource*);
u32 pci_read_config(void* pCardInfo, u32 offset, u32 len);
void pci_write_config(void* pCardInfo, u32 offset, u32 data, u32 len);

#endif //UNDER_RTSS
#endif /* _IGC_OS_H_ */
